@@ -1,7 +1,7 @@
 /* gdaui-data-cell-renderer-combo.c
  *
  * Copyright (C) 2000  Red Hat, Inc.,  Jonathan Blandford <jrb@redhat.com>
- * Copyright (C) 2003 - 2009 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2003 - 2010 Vivien Malerba <malerba@gnome-db.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -28,6 +28,7 @@
 #include <libgda-ui/gdaui-combo.h>
 #include <libgda-ui/gdaui-data-entry.h>
 #include <libgda/gda-enum-types.h>
+#include "gdaui-data-cell-renderer-util.h"
 
 #define GDAUI_DATA_CELL_RENDERER_COMBO_PATH "gdaui-data-cell-renderer-combo-path"
 
@@ -93,6 +94,7 @@ struct _GdauiDataCellRendererComboPrivate
 	gboolean      to_be_deleted;
 	gboolean      set_default_if_invalid;
 	gboolean      show_expander;
+	gboolean      invalid;
 };
 
 
@@ -115,6 +117,7 @@ gdaui_data_cell_renderer_combo_get_type (void)
 			sizeof (GdauiDataCellRendererCombo),
 			0,              /* n_preallocs */
 			(GInstanceInitFunc) gdaui_data_cell_renderer_combo_init,
+			0
 		};
 		
 		cell_text_type =
@@ -270,6 +273,7 @@ gdaui_data_cell_renderer_combo_set_property (GObject *object,
 
 	switch (param_id) {
 	case PROP_VALUES:
+		datacell->priv->invalid = FALSE;
 		if (value) {	
 			GList *gvalues = g_value_get_pointer (value);
 			if (gvalues) {
@@ -329,11 +333,15 @@ gdaui_data_cell_renderer_combo_set_property (GObject *object,
 
 				g_slist_free (values);
 			}
-			else
+			else {
+				datacell->priv->invalid = TRUE;
 				g_object_set (G_OBJECT (object), "text", "", NULL);
+			}
 		}
-		else
+		else {
+			datacell->priv->invalid = TRUE;
 			g_object_set (G_OBJECT (object), "text", "", NULL);
+		}
 		
 		g_object_notify (object, "values");
 		break;
@@ -342,7 +350,7 @@ gdaui_data_cell_renderer_combo_set_property (GObject *object,
 			GList *gvalues = g_value_get_pointer (value);
 			gchar *str;
 
-			g_assert (g_list_length (gvalues) == datacell->priv->source->shown_n_cols);
+			g_assert (g_list_length (gvalues) == (guint)datacell->priv->source->shown_n_cols);
 			str = render_text_to_display_from_values (gvalues);
 			g_object_set (G_OBJECT (object), "text", str, NULL);
 			g_free (str);
@@ -423,14 +431,14 @@ render_text_to_display_from_values (GList *values)
 }
 
 /**
- * gdaui_data_cell_renderer_combo_new
+ * gdaui_data_cell_renderer_combo_new:
  * @paramlist: a #GdaSet object
  * @source: a #GdauiSetSource structure listed in @paramlist->sources_list
  * 
  * Creates a new #GdauiDataCellRendererCombo which will fill the parameters listed in
  * @source->nodes with values available from @source->data_model.
  * 
- * Return value: the new cell renderer
+ * Return value: (transfer full): the new cell renderer
  **/
 GtkCellRenderer *
 gdaui_data_cell_renderer_combo_new (GdauiSet *paramlist, GdauiSetSource *source)
@@ -551,6 +559,9 @@ gdaui_data_cell_renderer_combo_render (GtkCellRenderer      *cell,
 				 cell_area->y + cell_area->height / 2.);
 		g_object_unref (style);
 	}
+
+	if (combocell->priv->invalid)
+		gdaui_data_cell_renderer_draw_invalid_area (window, cell_area);
 }
 
 static void gdaui_data_cell_renderer_combo_editing_done (GtkCellEditable *combo, GdauiDataCellRendererCombo *datacell);
@@ -559,12 +570,12 @@ static gboolean gdaui_data_cell_renderer_combo_focus_out_event (GtkWidget *widge
 
 static GtkCellEditable *
 gdaui_data_cell_renderer_combo_start_editing (GtkCellRenderer     *cell,
-					      GdkEvent            *event,
-					      GtkWidget           *widget,
+					      G_GNUC_UNUSED GdkEvent            *event,
+					      G_GNUC_UNUSED GtkWidget           *widget,
 					      const gchar         *path,
-					      GdkRectangle        *background_area,
-					      GdkRectangle        *cell_area,
-					      GtkCellRendererState flags)
+					      G_GNUC_UNUSED GdkRectangle        *background_area,
+					      G_GNUC_UNUSED GdkRectangle        *cell_area,
+					      G_GNUC_UNUSED GtkCellRendererState flags)
 {
 	GdauiDataCellRendererCombo *datacell;
 	GtkWidget *combo;
@@ -628,7 +639,7 @@ gdaui_data_cell_renderer_combo_editing_done (GtkCellEditable *combo, GdauiDataCe
 
 
 static gboolean
-gdaui_data_cell_renderer_combo_focus_out_event (GtkWidget *widget, GdkEvent  *event, 
+gdaui_data_cell_renderer_combo_focus_out_event (GtkWidget *widget, G_GNUC_UNUSED GdkEvent  *event,
 						GdauiDataCellRendererCombo *datacell)
 {
   
