@@ -473,7 +473,7 @@ gda_set_copy (GdaSet *set)
 }
 
 /**
- * gda_set_new_inline:
+ * gda_set_new_inline: (skip)
  * @nb: the number of value holders which will be contained in the new #GdaSet
  * @...: a serie of a (const gchar*) id, (GType) type, and value
  *
@@ -548,6 +548,10 @@ gda_set_new_inline (gint nb, ...)
 			g_value_set_ulong (value, va_arg (ap, gulong));
 		else if (type == G_TYPE_GTYPE)
 			g_value_set_gtype (value, va_arg(ap, GType));
+		else if (type == GDA_TYPE_TIMESTAMP)
+			gda_value_set_timestamp (value, va_arg(ap, GdaTimestamp *));
+		else if (type == GDA_TYPE_TIME)
+			gda_value_set_time (value, va_arg(ap, GdaTime *));
 		else {
 			g_warning (_("%s() does not handle values of type '%s'."),
 				   __FUNCTION__, g_type_name (type));
@@ -580,7 +584,7 @@ gda_set_new_inline (gint nb, ...)
 }
 
 /**
- * gda_set_set_holder_value:
+ * gda_set_set_holder_value: (skip)
  * @set: a #GdaSet object
  * @error: (allow-none): a place to store errors, or %NULL
  * @holder_id: the ID of the holder to set the value
