@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 Vivien Malerba
+ * Copyright (C) 2009 - 2011 Vivien Malerba
  *
  * This Library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public License as
@@ -30,6 +30,7 @@
 #include "browser-spinner.h"
 #include "browser-stock-icons.h"
 #include "connection-binding-properties.h"
+#include <gdk/gdkkeysyms.h>
 
 /*
  * structure representing a 'tab' in a window
@@ -59,6 +60,7 @@ static void browser_window_init (BrowserWindow *bwin);
 static void browser_window_dispose (GObject *object);
 
 static gboolean window_state_event (GtkWidget *widget, GdkEventWindowState *event);
+static gboolean key_press_event (GtkWidget *widget, GdkEventKey *event);
 static void connection_busy_cb (BrowserConnection *bcnc, gboolean is_busy, gchar *reason, BrowserWindow *bwin);
 
 static void connection_added_cb (BrowserCore *bcore, BrowserConnection *bcnc, BrowserWindow *bwin);
@@ -67,6 +69,14 @@ static void connection_removed_cb (BrowserCore *bcore, BrowserConnection *bcnc,
 static void transaction_status_changed_cb (BrowserConnection *bcnc, BrowserWindow *bwin);
 
 
+enum {
+        FULLSCREEN_CHANGED,
+        LAST_SIGNAL
+};
+
+static gint browser_window_signals[LAST_SIGNAL] = { 0 };
+
+
 /* get a pointer to the parents to be able to call their destructor */
 static GObjectClass  *parent_class = NULL;
 
@@ -77,6 +87,10 @@ struct _BrowserWindowPrivate {
 	PerspectiveData   *current_perspective;
 	guint              ui_manager_merge_id; /* for current perspective */
 
+	GtkWidget         *menubar;
+	GtkWidget         *toolbar;
+	gboolean           toolbar_shown;
+	gboolean           cursor_in_toolbar;
 	GtkWidget         *spinner;
 	GtkUIManager      *ui_manager;
 	GtkActionGroup    *agroup;
@@ -88,8 +102,17 @@ struct _BrowserWindowPrivate {
 	gulong             cnc_added_sigid;
 	gulong             cnc_removed_sigid;
 
+#if GTK_CHECK_VERSION (2,18,0)
+	GtkWidget         *notif_box;
+	GSList            *notif_widgets;
+#endif
+
 	GtkWidget         *statusbar;
 	guint              cnc_statusbar_context;
+
+	gboolean           fullscreen;
+	gulong             fullscreen_motion_sig_id;
+	guint              fullscreen_timer_id;
 };
 
 GType
@@ -108,7 +131,8 @@ browser_window_get_type (void)
 			NULL,
 			sizeof (BrowserWindow),
 			0,
-			(GInstanceInitFunc) browser_window_init
+			(GInstanceInitFunc) browser_window_init,
+			0
 		};
 
 		
@@ -125,11 +149,22 @@ browser_window_class_init (BrowserWindowClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
-	widget_class->window_state_event = window_state_event;
 
+	widget_class->window_state_event = window_state_event;
+	widget_class->key_press_event = key_press_event;
 	parent_class = g_type_class_peek_parent (klass);
 
+	browser_window_signals[FULLSCREEN_CHANGED] =
+                g_signal_new ("fullscreen_changed",
+                              G_TYPE_FROM_CLASS (object_class),
+                              G_SIGNAL_RUN_LAST,
+                              G_STRUCT_OFFSET (BrowserWindowClass, fullscreen_changed),
+                              NULL, NULL,
+                              g_cclosure_marshal_VOID__BOOLEAN,
+                              G_TYPE_NONE, 1, G_TYPE_BOOLEAN);
+
 	object_class->dispose = browser_window_dispose;
+	klass->fullscreen_changed = NULL;
 }
 
 static void
@@ -143,6 +178,9 @@ browser_window_init (BrowserWindow *bwin)
 	bwin->priv->cnc_added_sigid = 0;
 	bwin->priv->cnc_removed_sigid = 0;
 	bwin->priv->updating_transaction_status = FALSE;
+	bwin->priv->fullscreen = FALSE;
+	bwin->priv->fullscreen_motion_sig_id = 0;
+	bwin->priv->fullscreen_timer_id = 0;
 }
 
 static void
@@ -161,6 +199,12 @@ browser_window_dispose (GObject *object)
 			connection_removed_cb (browser_core_get(), BROWSER_CONNECTION (list->data), bwin);
 		g_slist_free (connections);
 
+		if (bwin->priv->fullscreen_timer_id)
+			g_source_remove (bwin->priv->fullscreen_timer_id);
+
+		if (bwin->priv->fullscreen_motion_sig_id)
+			g_signal_handler_disconnect (bwin, bwin->priv->fullscreen_motion_sig_id);
+
 		if (bwin->priv->cnc_added_sigid > 0)
 			g_signal_handler_disconnect (browser_core_get (), bwin->priv->cnc_added_sigid);
 		if (bwin->priv->cnc_removed_sigid > 0)
@@ -182,6 +226,11 @@ browser_window_dispose (GObject *object)
 		}
 		if (bwin->priv->perspectives_nb)
 			g_object_unref (bwin->priv->perspectives_nb);
+
+#if GTK_CHECK_VERSION (2,18,0)
+		if (bwin->priv->notif_widgets)
+			g_slist_free (bwin->priv->notif_widgets);
+#endif
 		g_free (bwin->priv);
 		bwin->priv = NULL;
 	}
@@ -192,7 +241,7 @@ browser_window_dispose (GObject *object)
 
 
 static gboolean
-delete_event (GtkWidget *widget, GdkEvent *event, BrowserWindow *bwin)
+delete_event (G_GNUC_UNUSED GtkWidget *widget, G_GNUC_UNUSED GdkEvent *event, BrowserWindow *bwin)
 {
 	browser_core_close_window (bwin);
         return TRUE;
@@ -224,26 +273,27 @@ static const GtkToggleActionEntry ui_toggle_actions [] =
 };
 
 static const GtkActionEntry ui_actions[] = {
-        { "Connection", NULL, "_Connection", NULL, "Connection", NULL },
-        { "ConnectionOpen", GTK_STOCK_CONNECT, "_Connect", NULL, "Open a connection", G_CALLBACK (connection_open_cb)},
+        { "Connection", NULL, N_("_Connection"), NULL, N_("Connection"), NULL },
+        { "ConnectionOpen", GTK_STOCK_CONNECT, N_("_Connect"), NULL, N_("Open a connection"), G_CALLBACK (connection_open_cb)},
         { "ConnectionBind", NULL, N_("_Bind connection"), "<control>I", N_("Use connection to create\n"
 						    "a new binding connection to access data\n"
 						    "from multiple databases at once"), G_CALLBACK (connection_bind_cb)},
-        { "ConnectionProps", GTK_STOCK_PROPERTIES, "_Properties", NULL, "Connection properties", G_CALLBACK (connection_properties_cb)},
-        { "ConnectionList", NULL, "_Connections list", NULL, "Connections list", G_CALLBACK (connection_list_cb)},
-        { "ConnectionMetaSync", GTK_STOCK_REFRESH, "_Fetch meta data", NULL, "Fetch meta data", G_CALLBACK (connection_meta_update_cb)},
-        { "ConnectionClose", GTK_STOCK_CLOSE, "_Close connection", NULL, "Close this connection", G_CALLBACK (connection_close_cb)},
-        { "Quit", GTK_STOCK_QUIT, "_Quit", NULL, "Quit", G_CALLBACK (quit_cb)},
-        { "Edit", NULL, "_Edit", NULL, "Edit", NULL },
-        { "Perspective", NULL, "_Perspective", NULL, "Perspective", NULL },
-        { "Window", NULL, "_Window", NULL, "Window", NULL },
-        { "WindowNew", STOCK_NEW_WINDOW, "_New window", "<control>N", "Open a new window for current connection", G_CALLBACK (window_new_cb)},
-        { "WindowNewOthers", NULL, "New window for _connection", NULL, "Open a new window for a connection", NULL},
-        { "WindowClose", GTK_STOCK_CLOSE, "_Close", "", "Close this window", G_CALLBACK (window_close_cb)},
-        { "Help", NULL, "_Help", NULL, "Help", NULL },
-        { "HelpAbout", GTK_STOCK_ABOUT, "_About", NULL, "About", G_CALLBACK (about_cb) },
+        { "ConnectionProps", GTK_STOCK_PROPERTIES, N_("_Properties"), NULL, N_("Connection properties"), G_CALLBACK (connection_properties_cb)},
+        { "ConnectionList", NULL, N_("_Connections list"), NULL, N_("Connections list"), G_CALLBACK (connection_list_cb)},
+        { "ConnectionMetaSync", GTK_STOCK_REFRESH, N_("_Fetch meta data"), NULL, N_("Fetch meta data"), G_CALLBACK (connection_meta_update_cb)},
+        { "ConnectionClose", GTK_STOCK_CLOSE, N_("_Close connection"), NULL, N_("Close this connection"), G_CALLBACK (connection_close_cb)},
+        { "Quit", GTK_STOCK_QUIT, N_("_Quit"), NULL, N_("Quit"), G_CALLBACK (quit_cb)},
+        { "Edit", NULL, N_("_Edit"), NULL, N_("Edit"), NULL },
+        { "Display", NULL, N_("_Display"), NULL, N_("Display"), NULL },
+        { "Perspective", NULL, N_("_Perspective"), NULL, N_("Perspective"), NULL },
+        { "Window", NULL, N_("_Window"), NULL, N_("Window"), NULL },
+        { "WindowNew", STOCK_NEW_WINDOW, N_("_New window"), "<control>N", N_("Open a new window for current connection"), G_CALLBACK (window_new_cb)},
+        { "WindowNewOthers", NULL, N_("New window for _connection"), NULL, N_("Open a new window for a connection"), NULL},
+        { "WindowClose", GTK_STOCK_CLOSE, N_("_Close"), NULL, N_("Close this window"), G_CALLBACK (window_close_cb)},
+        { "Help", NULL, N_("_Help"), NULL, N_("Help"), NULL },
+        { "HelpAbout", GTK_STOCK_ABOUT, N_("_About"), NULL, N_("About"), G_CALLBACK (about_cb) },
 #ifdef HAVE_GDU
-        { "HelpManual", GTK_STOCK_HELP, "_Manual", "F1", "Manual", G_CALLBACK (manual_cb) },
+        { "HelpManual", GTK_STOCK_HELP, N_("_Manual"), "F1", N_("Manual"), G_CALLBACK (manual_cb) },
 #endif
 	{ "TransactionBegin", BROWSER_STOCK_BEGIN, N_("Begin"), NULL, N_("Begin a new transaction"),
           G_CALLBACK (transaction_begin_cb)},
@@ -274,6 +324,8 @@ static const gchar *ui_actions_info =
         "    </menu>"
         "    <menu name='Edit' action='Edit'>"
         "    </menu>"
+        "    <menu name='Display' action='Display'>"
+        "    </menu>"
         "    <menu name='Perspective' action='Perspective'>"
         "      <placeholder name='PersList'/>"
         "    </menu>"
@@ -368,6 +420,8 @@ browser_window_new (BrowserConnection *bcnc, BrowserPerspectiveFactory *factory)
 	GtkActionGroup *group;
 
         group = gtk_action_group_new ("Actions");
+	gtk_action_group_set_translation_domain (group, GETTEXT_PACKAGE);
+
 	bwin->priv->agroup = group;
         gtk_action_group_add_actions (group, ui_actions, G_N_ELEMENTS (ui_actions), bwin);
 	gtk_action_group_add_toggle_actions (group, ui_toggle_actions, G_N_ELEMENTS (ui_toggle_actions), bwin);
@@ -383,6 +437,7 @@ browser_window_new (BrowserConnection *bcnc, BrowserPerspectiveFactory *factory)
 	gtk_window_add_accel_group (GTK_WINDOW (bwin), accel_group);
 
         menubar = gtk_ui_manager_get_widget (ui, "/MenuBar");
+	bwin->priv->menubar = menubar;
 #ifdef HAVE_MAC_INTEGRATION
 	gtk_osxapplication_set_menu_bar (theApp, GTK_MENU_SHELL (menubar));
 #else
@@ -391,11 +446,20 @@ browser_window_new (BrowserConnection *bcnc, BrowserPerspectiveFactory *factory)
 #endif
 
         toolbar = gtk_ui_manager_get_widget (ui, "/ToolBar");
+	bwin->priv->toolbar = toolbar;
+	bwin->priv->toolbar_shown = TRUE;
         gtk_box_pack_start (GTK_BOX (vbox), toolbar, FALSE, TRUE, 0);
 	gtk_toolbar_set_show_arrow (GTK_TOOLBAR (toolbar), TRUE);
         gtk_widget_show (toolbar);
 	bwin->priv->toolbar_style = gtk_toolbar_get_style (GTK_TOOLBAR (toolbar));
 
+#if GTK_CHECK_VERSION (2,18,0)
+	bwin->priv->notif_box = gtk_vbox_new (FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), bwin->priv->notif_box, FALSE, FALSE, 0);
+        gtk_widget_show (bwin->priv->notif_box);
+	bwin->priv->notif_widgets = NULL;
+#endif
+
 	GtkToolItem *ti;
 	GtkWidget *spinner, *svbox, *align;
 
@@ -422,9 +486,10 @@ browser_window_new (BrowserConnection *bcnc, BrowserPerspectiveFactory *factory)
 	/* statusbar */
 	bwin->priv->statusbar = gtk_statusbar_new ();
 
-
 	GSList *connections, *list;
 	bwin->priv->cnc_agroup = gtk_action_group_new ("CncActions");
+	gtk_action_group_set_translation_domain (bwin->priv->cnc_agroup, GETTEXT_PACKAGE);
+
 	connections = browser_core_get_connections ();
 	for (list = connections; list; list = list->next)
 		connection_added_cb (browser_core_get(), BROWSER_CONNECTION (list->data), bwin);
@@ -444,6 +509,7 @@ browser_window_new (BrowserConnection *bcnc, BrowserPerspectiveFactory *factory)
 	GtkActionGroup *actions;
 	actions = browser_perspective_get_actions_group (BROWSER_PERSPECTIVE (pers->perspective_widget));
 	if (actions) {
+		gtk_action_group_set_translation_domain (actions, GETTEXT_PACKAGE);
 		gtk_ui_manager_insert_action_group (bwin->priv->ui_manager, actions, 0);
 		g_object_unref (actions);
 	}
@@ -453,6 +519,12 @@ browser_window_new (BrowserConnection *bcnc, BrowserPerspectiveFactory *factory)
 		bwin->priv->ui_manager_merge_id = gtk_ui_manager_add_ui_from_string (bwin->priv->ui_manager,
 										     ui_info, -1, NULL);
 	bwin->priv->current_perspective = pers;
+	browser_perspective_get_current_customization (BROWSER_PERSPECTIVE (pers->perspective_widget),
+						       &actions, &ui_info);
+	browser_window_customize_perspective_ui (bwin, BROWSER_PERSPECTIVE (pers->perspective_widget),
+						 actions, ui_info);
+	if (actions)
+		g_object_unref (actions);
 	
 	/* insert perspective into window */
         bwin->priv->perspectives_nb = (GtkNotebook*) gtk_notebook_new ();
@@ -473,6 +545,8 @@ browser_window_new (BrowserConnection *bcnc, BrowserPerspectiveFactory *factory)
 
 	mid = gtk_ui_manager_new_merge_id (bwin->priv->ui_manager);
 	agroup = gtk_action_group_new ("Perspectives");
+	gtk_action_group_set_translation_domain (agroup, GETTEXT_PACKAGE);
+
 	gtk_ui_manager_insert_action_group (bwin->priv->ui_manager, agroup, 0);
 	bwin->priv->perspectives_actions = agroup;
 	g_object_unref (agroup);
@@ -518,6 +592,18 @@ browser_window_new (BrowserConnection *bcnc, BrowserPerspectiveFactory *factory)
 
         gtk_widget_show (GTK_WIDGET (bwin));
 
+#if GTK_CHECK_VERSION(2,18,0)
+	gtk_widget_set_can_focus ((GtkWidget* )pers->perspective_widget, TRUE);
+#else
+	GtkWidget *fwid = (GtkWidget* )pers->perspective_widget;
+	if (! GTK_WIDGET_CAN_FOCUS (fwid)) {
+		GTK_WIDGET_SET_FLAGS (fwid, GTK_CAN_FOCUS);
+		gtk_widget_queue_resize (fwid);
+		g_object_notify (G_OBJECT (fwid), "can-focus");
+	}
+#endif
+	gtk_widget_grab_focus ((GtkWidget* )pers->perspective_widget);
+
 	return bwin;
 }
 
@@ -556,11 +642,12 @@ perspective_toggle_cb (GtkRadioAction *action, GtkRadioAction *current, BrowserW
 		bwin->priv->perspectives = g_slist_prepend (bwin->priv->perspectives, pers);
 		pers->page_number = gtk_notebook_append_page (bwin->priv->perspectives_nb,
 							      GTK_WIDGET (pers->perspective_widget), NULL);
-		gtk_widget_show_all ((GtkWidget*) bwin->priv->perspectives_nb);
+		gtk_widget_show (GTK_WIDGET (pers->perspective_widget));
 
 		GtkActionGroup *actions;
 		actions = browser_perspective_get_actions_group (BROWSER_PERSPECTIVE (pers->perspective_widget));
 		if (actions) {
+			gtk_action_group_set_translation_domain (actions, GETTEXT_PACKAGE);
 			gtk_ui_manager_insert_action_group (bwin->priv->ui_manager, actions, 0);
 			g_object_unref (actions);
 		}
@@ -582,48 +669,55 @@ perspective_toggle_cb (GtkRadioAction *action, GtkRadioAction *current, BrowserW
 
 	/* current perspective's customizations */
 	bwin->priv->current_perspective = pers;
-	if (pers->customized_ui)
-		pers->customized_merge_id = gtk_ui_manager_add_ui_from_string (bwin->priv->ui_manager,
-									       pers->customized_ui,
-									       -1, NULL);
+
+	GtkActionGroup *actions;
+	browser_perspective_get_current_customization (BROWSER_PERSPECTIVE (pers->perspective_widget),
+						       &actions, &ui_info);
+	browser_window_customize_perspective_ui (bwin, BROWSER_PERSPECTIVE (pers->perspective_widget),
+						 actions, ui_info);
+	if (actions)
+		g_object_unref (actions);
 }
 
 static void
 connection_busy_cb (BrowserConnection *bcnc, gboolean is_busy, gchar *reason, BrowserWindow *bwin)
 {
-	if (is_busy) {
-		browser_spinner_start (BROWSER_SPINNER (bwin->priv->spinner));
-		gtk_widget_set_tooltip_text (bwin->priv->spinner, reason);
-		gtk_statusbar_push (GTK_STATUSBAR (bwin->priv->statusbar),
-				    bwin->priv->cnc_statusbar_context,
-				    reason);
-	}
-	else {
-		browser_spinner_stop (BROWSER_SPINNER (bwin->priv->spinner));
-		gtk_widget_set_tooltip_text (bwin->priv->spinner, NULL);
-		gtk_statusbar_pop (GTK_STATUSBAR (bwin->priv->statusbar),
-				   bwin->priv->cnc_statusbar_context);
-	}
-
 	GtkAction *action;
-	action = gtk_action_group_get_action (bwin->priv->agroup, "WindowNew");
-	gtk_action_set_sensitive (action, !is_busy);
-	action = gtk_action_group_get_action (bwin->priv->agroup, "ConnectionMetaSync");
-	gtk_action_set_sensitive (action, !is_busy);
+	if (bcnc == bwin->priv->bcnc) {
+		/* @bcbc is @bwin's own connection */
+		if (is_busy) {
+			browser_spinner_start (BROWSER_SPINNER (bwin->priv->spinner));
+			gtk_widget_set_tooltip_text (bwin->priv->spinner, reason);
+			gtk_statusbar_push (GTK_STATUSBAR (bwin->priv->statusbar),
+					    bwin->priv->cnc_statusbar_context,
+					    reason);
+		}
+		else {
+			browser_spinner_stop (BROWSER_SPINNER (bwin->priv->spinner));
+			gtk_widget_set_tooltip_text (bwin->priv->spinner, NULL);
+			gtk_statusbar_pop (GTK_STATUSBAR (bwin->priv->statusbar),
+					   bwin->priv->cnc_statusbar_context);
+		}
 
-	gboolean bsens = FALSE, csens = FALSE;
-	if (!is_busy) {
-		if (browser_connection_get_transaction_status (bcnc))
-			csens = TRUE;
-		else
-			bsens = TRUE;
+		gboolean bsens = FALSE, csens = FALSE;
+		if (!is_busy) {
+			if (browser_connection_get_transaction_status (bcnc))
+				csens = TRUE;
+			else
+				bsens = TRUE;
+		}
+		action = gtk_action_group_get_action (bwin->priv->agroup, "TransactionBegin");
+		gtk_action_set_sensitive (action, bsens);
+		action = gtk_action_group_get_action (bwin->priv->agroup, "TransactionCommit");
+		gtk_action_set_sensitive (action, csens);
+		action = gtk_action_group_get_action (bwin->priv->agroup, "TransactionRollback");
+		gtk_action_set_sensitive (action, csens);
+
+		action = gtk_action_group_get_action (bwin->priv->agroup, "WindowNew");
+		gtk_action_set_sensitive (action, !is_busy);
+		action = gtk_action_group_get_action (bwin->priv->agroup, "ConnectionMetaSync");
+		gtk_action_set_sensitive (action, !is_busy);
 	}
-	action = gtk_action_group_get_action (bwin->priv->agroup, "TransactionBegin");
-	gtk_action_set_sensitive (action, bsens);
-	action = gtk_action_group_get_action (bwin->priv->agroup, "TransactionCommit");
-	gtk_action_set_sensitive (action, csens);
-	action = gtk_action_group_get_action (bwin->priv->agroup, "TransactionRollback");
-	gtk_action_set_sensitive (action, csens);	
 
 	const gchar *cncname;
 	cncname = browser_connection_get_name (bcnc);
@@ -634,7 +728,7 @@ connection_busy_cb (BrowserConnection *bcnc, gboolean is_busy, gchar *reason, Br
 
 /* update @bwin->priv->cnc_agroup and @bwin->priv->ui_manager */
 static void
-connection_added_cb (BrowserCore *bcore, BrowserConnection *bcnc, BrowserWindow *bwin)
+connection_added_cb (G_GNUC_UNUSED BrowserCore *bcore, BrowserConnection *bcnc, BrowserWindow *bwin)
 {
 	GtkAction *action;
 	const gchar *cncname;
@@ -668,7 +762,7 @@ connection_added_cb (BrowserCore *bcore, BrowserConnection *bcnc, BrowserWindow
 
 /* update @bwin->priv->cnc_agroup and @bwin->priv->ui_manager */
 static void
-connection_removed_cb (BrowserCore *bcore, BrowserConnection *bcnc, BrowserWindow *bwin)
+connection_removed_cb (G_GNUC_UNUSED BrowserCore *bcore, BrowserConnection *bcnc, BrowserWindow *bwin)
 {
 	GtkAction *action;
 	gchar *path;
@@ -692,7 +786,7 @@ connection_removed_cb (BrowserCore *bcore, BrowserConnection *bcnc, BrowserWindo
 }
 
 static void
-connection_close_cb (GtkAction *action, BrowserWindow *bwin)
+connection_close_cb (G_GNUC_UNUSED GtkAction *action, BrowserWindow *bwin)
 {
 	/* confirmation dialog */
 	GtkWidget *dialog;
@@ -730,7 +824,7 @@ connection_close_cb (GtkAction *action, BrowserWindow *bwin)
 }
 
 static void
-quit_cb (GtkAction *action, BrowserWindow *bwin)
+quit_cb (G_GNUC_UNUSED GtkAction *action, BrowserWindow *bwin)
 {
 	/* confirmation dialog */
 	GtkWidget *dialog;
@@ -795,7 +889,7 @@ transaction_status_changed_cb (BrowserConnection *bcnc, BrowserWindow *bwin)
 }
 
 static void
-transaction_begin_cb (GtkAction *action, BrowserWindow *bwin)
+transaction_begin_cb (G_GNUC_UNUSED GtkAction *action, BrowserWindow *bwin)
 {
 	if (!bwin->priv->updating_transaction_status) {
 		GError *error = NULL;
@@ -809,7 +903,7 @@ transaction_begin_cb (GtkAction *action, BrowserWindow *bwin)
 }
 
 static void
-transaction_commit_cb (GtkAction *action, BrowserWindow *bwin)
+transaction_commit_cb (G_GNUC_UNUSED GtkAction *action, BrowserWindow *bwin)
 {
 	if (!bwin->priv->updating_transaction_status) {
 		GError *error = NULL;
@@ -823,7 +917,7 @@ transaction_commit_cb (GtkAction *action, BrowserWindow *bwin)
 }
 
 static void
-transaction_rollback_cb (GtkAction *action, BrowserWindow *bwin)
+transaction_rollback_cb (G_GNUC_UNUSED GtkAction *action, BrowserWindow *bwin)
 {
 	if (!bwin->priv->updating_transaction_status) {
 		GError *error = NULL;
@@ -838,18 +932,133 @@ transaction_rollback_cb (GtkAction *action, BrowserWindow *bwin)
 
 
 static void
-window_close_cb (GtkAction *action, BrowserWindow *bwin)
+window_close_cb (G_GNUC_UNUSED GtkAction *action, BrowserWindow *bwin)
 {
 	delete_event (NULL, NULL, bwin);
 }
 
+static gboolean
+toolbar_hide_timeout_cb (BrowserWindow *bwin)
+{
+	if (!bwin->priv->cursor_in_toolbar) {
+		gtk_widget_hide (bwin->priv->toolbar);
+		gtk_widget_hide (bwin->priv->menubar);
+		bwin->priv->toolbar_shown = FALSE;
+		
+		/* remove timer */
+		bwin->priv->fullscreen_timer_id = 0;
+		return FALSE;
+	}
+	else
+		/* keep timer */
+		return TRUE;
+}
+
+#define BWIN_WINDOW_FULLSCREEN_POPUP_THRESHOLD 5
+#define BWIN_WINDOW_FULLSCREEN_POPUP_TIMER 1
+
+static gboolean
+fullscreen_motion_notify_cb (GtkWidget *widget, GdkEventMotion *event, G_GNUC_UNUSED gpointer user_data)
+{
+	BrowserWindow *bwin = BROWSER_WINDOW (widget);
+#if GTK_CHECK_VERSION(2,14,0)
+	if (gtk_widget_get_window (widget) != event->window)
+		return FALSE;
+#else
+	if (widget->window != event->window)
+		return FALSE;
+#endif
+
+	if (event->y < BWIN_WINDOW_FULLSCREEN_POPUP_THRESHOLD) {
+		gtk_widget_show (bwin->priv->toolbar);
+		gtk_widget_show (bwin->priv->menubar);
+		bwin->priv->toolbar_shown = TRUE;
+	}
+
+	if (bwin->priv->toolbar_shown) {
+		/* reset toolbar hiding timer */
+		if (bwin->priv->fullscreen_timer_id)
+			g_source_remove (bwin->priv->fullscreen_timer_id);
+		bwin->priv->fullscreen_timer_id = g_timeout_add_seconds (BWIN_WINDOW_FULLSCREEN_POPUP_TIMER,
+									 (GSourceFunc) toolbar_hide_timeout_cb,
+									 bwin);
+	}
+	return FALSE;
+}
+
+gboolean
+toolbar_enter_notify_cb (G_GNUC_UNUSED GtkWidget *widget, G_GNUC_UNUSED GdkEventCrossing *event,
+			 BrowserWindow *bwin)
+{
+	bwin->priv->cursor_in_toolbar = TRUE;
+	return FALSE;
+}
+
+gboolean
+toolbar_leave_notify_cb (G_GNUC_UNUSED GtkWidget *widget, G_GNUC_UNUSED GdkEventCrossing *event, BrowserWindow *bwin)
+{
+	bwin->priv->cursor_in_toolbar = FALSE;
+	return FALSE;
+}
+
 static void
 window_fullscreen_cb (GtkToggleAction *action, BrowserWindow *bwin)
 {
-	if (gtk_toggle_action_get_active (action))
+	if (gtk_toggle_action_get_active (action)) {
 		gtk_window_fullscreen (GTK_WINDOW (bwin));
-	else
+		browser_window_show_notice_printf (bwin, GTK_MESSAGE_INFO,
+						   "fullscreen-esc",
+						   "%s", _("Hit the Escape key to leave the fullscreen mode"));
+		gtk_widget_hide (bwin->priv->toolbar);
+		gtk_widget_hide (bwin->priv->menubar);
+		bwin->priv->toolbar_shown = FALSE;
+		bwin->priv->fullscreen_motion_sig_id = g_signal_connect (bwin, "motion-notify-event",
+									 G_CALLBACK (fullscreen_motion_notify_cb),
+									 NULL);
+		g_signal_connect (bwin->priv->toolbar, "enter-notify-event",
+				  G_CALLBACK (toolbar_enter_notify_cb), bwin);
+		g_signal_connect (bwin->priv->toolbar, "leave-notify-event",
+				  G_CALLBACK (toolbar_leave_notify_cb), bwin);
+		g_signal_connect (bwin->priv->menubar, "enter-notify-event",
+				  G_CALLBACK (toolbar_enter_notify_cb), bwin);
+		g_signal_connect (bwin->priv->menubar, "leave-notify-event",
+				  G_CALLBACK (toolbar_leave_notify_cb), bwin);
+	}
+	else {
 		gtk_window_unfullscreen (GTK_WINDOW (bwin));
+		g_signal_handler_disconnect (bwin, bwin->priv->fullscreen_motion_sig_id);
+		bwin->priv->fullscreen_motion_sig_id = 0;
+		g_signal_handlers_disconnect_by_func (bwin->priv->toolbar,
+						      G_CALLBACK (toolbar_enter_notify_cb), bwin);
+		g_signal_handlers_disconnect_by_func (bwin->priv->toolbar,
+						      G_CALLBACK (toolbar_leave_notify_cb), bwin);
+		g_signal_handlers_disconnect_by_func (bwin->priv->menubar,
+						      G_CALLBACK (toolbar_enter_notify_cb), bwin);
+		g_signal_handlers_disconnect_by_func (bwin->priv->menubar,
+						      G_CALLBACK (toolbar_leave_notify_cb), bwin);
+
+		gtk_widget_show (bwin->priv->toolbar);
+		gtk_widget_show (bwin->priv->menubar);
+		bwin->priv->toolbar_shown = TRUE;
+
+		if (bwin->priv->fullscreen_timer_id) {
+			g_source_remove (bwin->priv->fullscreen_timer_id);
+			bwin->priv->fullscreen_timer_id = 0;
+		}
+	}
+}
+
+static gboolean
+key_press_event (GtkWidget *widget, GdkEventKey *event)
+{
+	if ((event->keyval == GDK_Escape) &&
+	    browser_window_is_fullscreen (BROWSER_WINDOW (widget))) {
+		browser_window_set_fullscreen (BROWSER_WINDOW (widget), FALSE);
+		return TRUE;
+	}
+
+	/* parent class */
+	return GTK_WIDGET_CLASS (parent_class)->key_press_event (widget, event);
 }
 
 static gboolean
@@ -870,13 +1079,16 @@ window_state_event (GtkWidget *widget, GdkEventWindowState *event)
 		wid = gtk_ui_manager_get_widget (bwin->priv->ui_manager, "/ToolBar");
 
                 fullscreen = event->new_window_state & GDK_WINDOW_STATE_FULLSCREEN;
+		bwin->priv->fullscreen = fullscreen;
 		if (fullscreen) {
 			gtk_toolbar_set_style (GTK_TOOLBAR (wid), GTK_TOOLBAR_ICONS);
-			browser_spinner_set_size (BROWSER_SPINNER (bwin->priv->spinner), GTK_ICON_SIZE_LARGE_TOOLBAR);
+			browser_spinner_set_size (BROWSER_SPINNER (bwin->priv->spinner),
+						  GTK_ICON_SIZE_LARGE_TOOLBAR);
 		}
 		else {
 			gtk_toolbar_set_style (GTK_TOOLBAR (wid), bwin->priv->toolbar_style);
-			browser_spinner_set_size (BROWSER_SPINNER (bwin->priv->spinner), GTK_ICON_SIZE_DIALOG);
+			browser_spinner_set_size (BROWSER_SPINNER (bwin->priv->spinner),
+						  GTK_ICON_SIZE_SMALL_TOOLBAR);
 		}
 
 		wid = gtk_ui_manager_get_widget (bwin->priv->ui_manager, "/MenuBar");
@@ -886,12 +1098,13 @@ window_state_event (GtkWidget *widget, GdkEventWindowState *event)
 			gtk_widget_show (wid);
 		
 		gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (bwin->priv->statusbar), !fullscreen);
+		g_signal_emit (G_OBJECT (bwin), browser_window_signals[FULLSCREEN_CHANGED], 0, fullscreen);
         }
 	return FALSE;
 }
 
 static void
-window_new_cb (GtkAction *action, BrowserWindow *bwin)
+window_new_cb (G_GNUC_UNUSED GtkAction *action, BrowserWindow *bwin)
 {
 	BrowserWindow *nbwin;
 	BrowserConnection *bcnc;
@@ -902,7 +1115,7 @@ window_new_cb (GtkAction *action, BrowserWindow *bwin)
 }
 
 static void
-window_new_with_cnc_cb (GtkAction *action, BrowserWindow *bwin)
+window_new_with_cnc_cb (GtkAction *action, G_GNUC_UNUSED BrowserWindow *bwin)
 {
 	BrowserWindow *nbwin;
 	BrowserConnection *bcnc;
@@ -915,13 +1128,13 @@ window_new_with_cnc_cb (GtkAction *action, BrowserWindow *bwin)
 }
 
 static void
-connection_open_cb (GtkAction *action, BrowserWindow *bwin)
+connection_open_cb (G_GNUC_UNUSED GtkAction *action, G_GNUC_UNUSED BrowserWindow *bwin)
 {
 	browser_connection_open (NULL);
 }
 
 static void
-connection_properties_cb (GtkAction *action, BrowserWindow *bwin)
+connection_properties_cb (G_GNUC_UNUSED GtkAction *action, BrowserWindow *bwin)
 {
 	if (BROWSER_IS_VIRTUAL_CONNECTION (bwin->priv->bcnc)) {
 		GtkWidget *win;
@@ -938,7 +1151,7 @@ connection_properties_cb (GtkAction *action, BrowserWindow *bwin)
 		if (res == GTK_RESPONSE_OK) {
 			GError *error = NULL;
 			if (!browser_virtual_connection_modify_specs (BROWSER_VIRTUAL_CONNECTION (bwin->priv->bcnc),
-					connection_binding_properties_get_specs (CONNECTION_BINDING_PROPERTIES (win)),
+								      connection_binding_properties_get_specs (CONNECTION_BINDING_PROPERTIES (win)),
 								      &error)) {
 				browser_show_error ((GtkWindow*) bwin,
 						    _("Error updating bound connection: %s"),
@@ -957,7 +1170,7 @@ connection_properties_cb (GtkAction *action, BrowserWindow *bwin)
 }
 
 static void
-connection_bind_cb (GtkAction *action, BrowserWindow *bwin)
+connection_bind_cb (G_GNUC_UNUSED GtkAction *action, BrowserWindow *bwin)
 {
 	GtkWidget *win;
 	gint res;
@@ -991,19 +1204,19 @@ connection_bind_cb (GtkAction *action, BrowserWindow *bwin)
 }
 
 static void
-connection_list_cb (GtkAction *action, BrowserWindow *bwin)
+connection_list_cb (G_GNUC_UNUSED GtkAction *action, BrowserWindow *bwin)
 {
 	browser_connections_list_show (bwin->priv->bcnc);
 }
 
 static void
-connection_meta_update_cb (GtkAction *action, BrowserWindow *bwin)
+connection_meta_update_cb (G_GNUC_UNUSED GtkAction *action, BrowserWindow *bwin)
 {
 	browser_connection_update_meta_data (bwin->priv->bcnc);
 }
 
 static void
-about_cb (GtkAction *action, BrowserWindow *bwin)
+about_cb (G_GNUC_UNUSED GtkAction *action, BrowserWindow *bwin)
 {
 	GdkPixbuf *icon;
         GtkWidget *dialog;
@@ -1024,7 +1237,7 @@ about_cb (GtkAction *action, BrowserWindow *bwin)
         dialog = gtk_about_dialog_new ();
 	gtk_about_dialog_set_program_name (GTK_ABOUT_DIALOG (dialog), _("Database browser"));
         gtk_about_dialog_set_version (GTK_ABOUT_DIALOG (dialog), PACKAGE_VERSION);
-        gtk_about_dialog_set_copyright (GTK_ABOUT_DIALOG (dialog), "(C) 2009 GNOME Foundation");
+        gtk_about_dialog_set_copyright (GTK_ABOUT_DIALOG (dialog), "(C) 2009 - 2011 GNOME Foundation");
 	gtk_about_dialog_set_comments (GTK_ABOUT_DIALOG (dialog), _("Database access services for the GNOME Desktop"));
         gtk_about_dialog_set_license (GTK_ABOUT_DIALOG (dialog), "GNU General Public License");
         gtk_about_dialog_set_website (GTK_ABOUT_DIALOG (dialog), "http://www.gnome-db.org");
@@ -1041,7 +1254,7 @@ about_cb (GtkAction *action, BrowserWindow *bwin)
 
 #ifdef HAVE_GDU
 void
-manual_cb (GtkAction *action, BrowserWindow *bwin)
+manual_cb (G_GNUC_UNUSED GtkAction *action, BrowserWindow *bwin)
 {
 	browser_show_help (GTK_WINDOW (bwin), NULL);
 }
@@ -1176,6 +1389,228 @@ browser_window_pop_status (BrowserWindow *bwin, const gchar *context)
 	gtk_statusbar_pop (GTK_STATUSBAR (bwin->priv->statusbar), cid);
 }
 
+/**
+ * browser_window_show_notice_printf
+ * @bwin: a #BrowserWindow
+ * @context: textual description of what context the message is being used in
+ * @format: the text to display
+ *
+ * Make @bwin display a notice
+ */
+void
+browser_window_show_notice_printf (BrowserWindow *bwin, GtkMessageType type, const gchar *context,
+				   const gchar *format, ...)
+{
+	va_list args;
+        gchar sz[2048];
+
+	g_return_if_fail (BROWSER_IS_WINDOW (bwin));
+
+        /* build the message string */
+        va_start (args, format);
+        vsnprintf (sz, sizeof (sz), format, args);
+        va_end (args);
+	browser_window_show_notice (bwin, type, context, sz);
+}
+
+
+#if GTK_CHECK_VERSION (2,18,0)
+static void
+info_bar_response_cb (GtkInfoBar *ibar, G_GNUC_UNUSED gint response, BrowserWindow *bwin)
+{
+	bwin->priv->notif_widgets = g_slist_remove (bwin->priv->notif_widgets, ibar);	
+	gtk_widget_destroy ((GtkWidget*) ibar);
+}
+#endif
+
+/* hash table to remain which context notices have to be hidden: key=context, value=GINT_TO_POINTER (1) */
+static GHashTable *hidden_contexts = NULL;
+static void
+hidden_contexts_foreach_save (const gchar *context, G_GNUC_UNUSED gint value, xmlNodePtr root)
+{
+	xmlNodePtr node;
+	node = xmlNewChild (root, NULL, BAD_CAST "hide-notice", BAD_CAST context);
+}
+
+static void
+hide_notice_toggled_cb (GtkToggleButton *toggle, gchar *context)
+{
+	g_assert (hidden_contexts);
+	if (gtk_toggle_button_get_active (toggle))
+		g_hash_table_insert (hidden_contexts, g_strdup (context), GINT_TO_POINTER (TRUE));
+	else
+		g_hash_table_remove (hidden_contexts, context);
+
+	/* save config */
+	xmlDocPtr doc;
+	xmlNodePtr root;
+	xmlChar *xml_contents;
+	gint size;
+	doc = xmlNewDoc (BAD_CAST "1.0");
+	root = xmlNewNode (NULL, BAD_CAST "gda-browser-preferences");
+	xmlDocSetRootElement (doc, root);
+	g_hash_table_foreach (hidden_contexts, (GHFunc) hidden_contexts_foreach_save, root);
+	xmlDocDumpFormatMemory (doc, &xml_contents, &size, 1);
+	xmlFreeDoc (doc);
+
+	/* save to disk */
+	gchar *confdir, *conffile = NULL;
+	GError *lerror = NULL;
+	confdir = g_build_path (G_DIR_SEPARATOR_S, g_get_user_config_dir(), "gda-browser", NULL);
+	if (!g_file_test (confdir, G_FILE_TEST_EXISTS)) {
+		if (g_mkdir_with_parents (confdir, 0700)) {
+			g_warning ("Can't create configuration directory '%s' to save preferences.",
+				   confdir);
+			goto out;
+		}
+	}
+	conffile = g_build_filename (confdir, "preferences.xml", NULL);
+	if (! g_file_set_contents (conffile, (gchar *) xml_contents, size, &lerror)) {
+		g_warning ("Can't save preferences file '%s': %s", conffile,
+			   lerror && lerror->message ? lerror->message : _("No detail"));
+		g_clear_error (&lerror);
+	}
+
+ out:
+	xmlFree (xml_contents);
+	g_free (confdir);
+	g_free (conffile);
+}
+
+static void
+load_preferences (void)
+{
+	/* load preferences */
+	xmlDocPtr doc;
+	xmlNodePtr root, node;
+	gchar *conffile;
+	conffile = g_build_path (G_DIR_SEPARATOR_S, g_get_user_config_dir(),
+				 "gda-browser", "preferences.xml", NULL);
+	if (!g_file_test (conffile, G_FILE_TEST_EXISTS)) {
+		g_free (conffile);
+		return;
+	}
+
+	doc = xmlParseFile (conffile);
+	if (!doc) {
+		g_warning ("Error loading preferences from file '%s'", conffile);
+		g_free (conffile);
+		return;
+	}
+	g_free (conffile);
+
+	root = xmlDocGetRootElement (doc);
+	for (node = root->children; node; node = node->next) {
+		xmlChar *contents;
+		if (strcmp ((gchar *) node->name, "hide-notice"))
+			continue;
+		contents = xmlNodeGetContent (node);
+		if (contents) {
+			g_hash_table_insert (hidden_contexts, g_strdup ((gchar*) contents),
+					     GINT_TO_POINTER (TRUE));
+			xmlFree (contents);
+		}
+	}
+	xmlFreeDoc (doc);
+}
+
+
+/**
+ * browser_window_show_notice
+ * @bwin: a #BrowserWindow
+ * @context: textual description of what context the message is being used in
+ * @text: the information's text
+ *
+ * Makes @bwin display a notice
+ */
+void
+browser_window_show_notice (BrowserWindow *bwin, GtkMessageType type, const gchar *context, const gchar *text)
+{
+	g_return_if_fail (BROWSER_IS_WINDOW (bwin));
+	gboolean hide = FALSE;
+
+	if ((type != GTK_MESSAGE_ERROR) && context) {
+		if (!hidden_contexts) {
+			hidden_contexts = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
+			load_preferences ();
+		}
+		hide = GPOINTER_TO_INT (g_hash_table_lookup (hidden_contexts, context));
+	}
+
+	if (hide) {
+		gchar *ptr, *tmp;
+		tmp = g_strdup (text);
+		for (ptr = tmp; *ptr && (*ptr != '\n'); ptr++);
+		if (*ptr) {
+			*ptr = '.'; ptr++;
+			*ptr = '.'; ptr++;
+			*ptr = '.'; ptr++;
+			*ptr = 0;
+		}
+		browser_window_push_status (bwin, "SupportNotice", tmp, TRUE);
+		g_free (tmp);
+	}
+	else {
+		GtkWidget *cb = NULL;
+		if (context && (type == GTK_MESSAGE_INFO)) {
+			cb = gtk_check_button_new_with_label (_("Don't show this message again"));
+			g_signal_connect_data (cb, "toggled",
+					       G_CALLBACK (hide_notice_toggled_cb), g_strdup (context),
+					       (GClosureNotify) g_free, 0);
+		}
+
+#if GTK_CHECK_VERSION (2,18,0)
+		/* use a GtkInfoBar */
+		GtkWidget *ibar, *content_area, *label;
+		
+		ibar = gtk_info_bar_new_with_buttons (GTK_STOCK_CLOSE, 1, NULL);
+		gtk_info_bar_set_message_type (GTK_INFO_BAR (ibar), type);
+		label = gtk_label_new ("");
+		gtk_label_set_markup (GTK_LABEL (label), text);
+		gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
+		gtk_misc_set_alignment (GTK_MISC (label), 0., -1);
+		content_area = gtk_info_bar_get_content_area (GTK_INFO_BAR (ibar));
+		if (cb) {
+			GtkWidget *box;
+			box = gtk_hbox_new (FALSE, 0);
+			gtk_box_pack_start (GTK_BOX (box), label, TRUE, TRUE, 0);
+			gtk_box_pack_start (GTK_BOX (box), cb, FALSE, FALSE, 0);
+			gtk_container_add (GTK_CONTAINER (content_area), box);
+			gtk_widget_show_all (box);
+		}
+		else {
+			gtk_container_add (GTK_CONTAINER (content_area), label);
+			gtk_widget_show (label);
+		}
+		g_signal_connect (ibar, "response",
+				  G_CALLBACK (info_bar_response_cb), bwin);
+		gtk_box_pack_start (GTK_BOX (bwin->priv->notif_box), ibar, TRUE, TRUE, 0);
+		bwin->priv->notif_widgets = g_slist_append (bwin->priv->notif_widgets, ibar);
+		if (g_slist_length (bwin->priv->notif_widgets) > 2) {
+			gtk_widget_destroy (GTK_WIDGET (bwin->priv->notif_widgets->data));
+			bwin->priv->notif_widgets = g_slist_delete_link (bwin->priv->notif_widgets,
+									 bwin->priv->notif_widgets);
+		}
+		gtk_widget_show (ibar);
+#else
+		/* create the error message dialog */
+		GtkWidget *dialog;
+		dialog = gtk_message_dialog_new_with_markup (GTK_WINDOW (bwin),
+							     GTK_DIALOG_DESTROY_WITH_PARENT |
+							     GTK_DIALOG_MODAL, GTK_MESSAGE_INFO,
+							     GTK_BUTTONS_CLOSE,
+							     "<span weight=\"bold\">%s</span>\n%s", _("Note:"), text);
+		if (cb)
+			gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), cb, FALSE, FALSE, 10);
+	
+		gtk_widget_show_all (dialog);
+		gtk_dialog_run (GTK_DIALOG (dialog));
+		gtk_widget_destroy (dialog);
+#endif
+	}
+}
+
+
 /**
  * browser_window_customize_perspective_ui
  * @bwin: a #BrowserWindow
@@ -1224,6 +1659,7 @@ browser_window_customize_perspective_ui (BrowserWindow *bwin, BrowserPerspective
 
 	if (actions_group) {
 		g_return_if_fail (GTK_IS_ACTION_GROUP (actions_group));
+		gtk_action_group_set_translation_domain (actions_group, GETTEXT_PACKAGE);
 		gtk_ui_manager_insert_action_group (bwin->priv->ui_manager, actions_group, 0);
 		pdata->customized_actions = g_object_ref (actions_group);
 	}
@@ -1275,13 +1711,47 @@ browser_window_change_perspective (BrowserWindow *bwin, const gchar *perspective
 	}
 	g_list_free (actions);
 
-	browser_show_notice (GTK_WINDOW (bwin), "Perspective change",
-			     _("The current perspective has changed to the '%s' perspective, you "
-			       "can switch back to previous perspective through the "
-			       "'Perspective/%s' menu, or using the '%s' shortcut"),
-			     bwin->priv->current_perspective->factory->perspective_name,
-			     current_pdata->factory->perspective_name,
-			     current_pdata->factory->menu_shortcut);
+	gchar *tmp;
+	tmp = g_markup_printf_escaped (_("The current perspective has changed to the '%s' perspective, you "
+					 "can switch back to previous perspective through the "
+					 "'Perspective/%s' menu, or using the '%s' shortcut"),
+				       bwin->priv->current_perspective->factory->perspective_name,
+				       current_pdata->factory->perspective_name,
+				       current_pdata->factory->menu_shortcut);
+
+			
+	browser_window_show_notice (bwin, GTK_MESSAGE_INFO, "Perspective change", tmp);
+	g_free (tmp);
 
 	return bpers;
 }
+
+/**
+ * browser_window_is_fullscreen
+ * @bwin: a #BrowserWindow
+ *
+ * Returns: %TRUE if @bwin is fullscreen
+ */
+gboolean
+browser_window_is_fullscreen (BrowserWindow *bwin)
+{
+	g_return_val_if_fail (BROWSER_IS_WINDOW (bwin), FALSE);
+	return bwin->priv->fullscreen;
+}
+
+/**
+ * browser_window_set_fullscreen
+ * @bwin: a #BrowserWindow
+ * @fullscreen:
+ *
+ * Requires @bwin to be fullscreen if @fullscreen is %TRUE
+ */
+void
+browser_window_set_fullscreen (BrowserWindow *bwin, gboolean fullscreen)
+{
+	GtkAction *action;
+	g_return_if_fail (BROWSER_IS_WINDOW (bwin));
+	
+	action = gtk_action_group_get_action (bwin->priv->agroup, "WindowFullScreen");
+	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), fullscreen);
+}
