@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 - 2010 The GNOME Foundation
+ * Copyright (C) 2009 - 2011 The GNOME Foundation
  *
  * AUTHORS:
  *      Vivien Malerba <malerba@gnome-db.org>
@@ -33,17 +33,18 @@
 #include "../browser-page.h"
 #include "../browser-perspective.h"
 #include "../browser-stock-icons.h"
-#include "../common/popup-container.h"
+#include <libgda-ui/internal/popup-container.h>
 #include <libgda/sql-parser/gda-sql-parser.h>
 #include <libgda-ui/libgda-ui.h>
 #include "data-source-manager.h"
 #include <gdk/gdkkeysyms.h>
+#include "analyser.h"
 
 #define MAIN_PAGE_EDITORS 0
 #define MAIN_PAGE_DATA 1
 
-#define EDITOR_PAGE_XML 0
-#define EDITOR_PAGE_UI 1
+#define EDITOR_PAGE_XML 1
+#define EDITOR_PAGE_UI 0
 
 typedef enum {
 	LAYOUT_HORIZ,
@@ -84,6 +85,7 @@ struct _DataConsolePrivate {
 	GtkWidget *add_source_button;
 	GtkWidget *add_source_menu;
 	gpointer   add_source_menu_index; /* used to know if @add_source_menu needs to be rebuilt */
+	GSList    *customized_menu_items_list; /* list of GtkMenuItem in @add_source_menu */
 };
 
 static void data_console_class_init (DataConsoleClass *klass);
@@ -169,7 +171,7 @@ data_console_page_init (BrowserPageIface *iface)
 }
 
 static void
-data_console_init (DataConsole *dconsole, DataConsoleClass *klass)
+data_console_init (DataConsole *dconsole, G_GNUC_UNUSED DataConsoleClass *klass)
 {
 	dconsole->priv = g_new0 (DataConsolePrivate, 1);
 	dconsole->priv->layout_type = LAYOUT_HORIZ;
@@ -186,6 +188,8 @@ data_console_dispose (GObject *object)
 	if (dconsole->priv) {
 		if (dconsole->priv->add_source_menu)
 			gtk_widget_destroy (dconsole->priv->add_source_menu);
+		if (dconsole->priv->customized_menu_items_list)
+			g_slist_free (dconsole->priv->customized_menu_items_list);
 		if (dconsole->priv->params_form)
 			gtk_widget_destroy (dconsole->priv->params_form);
 		if (dconsole->priv->popup_container)
@@ -225,7 +229,8 @@ data_console_get_type (void)
 			NULL,
 			sizeof (DataConsole),
 			0,
-			(GInstanceInitFunc) data_console_init
+			(GInstanceInitFunc) data_console_init,
+			0
 		};
 
 		static GInterfaceInfo page_console = {
@@ -365,49 +370,49 @@ data_console_new (BrowserConnection *bcnc)
 	gtk_notebook_set_show_border (GTK_NOTEBOOK (nb), FALSE);
 	dconsole->priv->editors_notebook = nb;
 
-	dconsole->priv->xml_sped = xml_spec_editor_new (dconsole->priv->mgr);
-	gtk_widget_show (dconsole->priv->xml_sped);
-	gtk_notebook_append_page (GTK_NOTEBOOK (dconsole->priv->editors_notebook),
-				  dconsole->priv->xml_sped, NULL);
-
 	dconsole->priv->ui_sped = ui_spec_editor_new (dconsole->priv->mgr);
 	gtk_widget_show (dconsole->priv->ui_sped);
 	gtk_notebook_append_page (GTK_NOTEBOOK (dconsole->priv->editors_notebook),
 				  dconsole->priv->ui_sped, NULL);
 
+	dconsole->priv->xml_sped = xml_spec_editor_new (dconsole->priv->mgr);
+	gtk_widget_show (dconsole->priv->xml_sped);
+	gtk_notebook_append_page (GTK_NOTEBOOK (dconsole->priv->editors_notebook),
+				  dconsole->priv->xml_sped, NULL);
+
 	/* buttons */
 	GtkWidget *bbox, *button;
 	bbox = gtk_vbutton_box_new ();
 	gtk_button_box_set_layout (GTK_BUTTON_BOX (bbox), GTK_BUTTONBOX_END);
 	gtk_box_pack_start (GTK_BOX (hbox), bbox, FALSE, FALSE, 5);
 
-	button = browser_make_small_button (FALSE, _("Reset"), GTK_STOCK_CLEAR,
+	button = browser_make_small_button (FALSE, FALSE, _("Reset"), GTK_STOCK_CLEAR,
 					    _("Reset the editor's\ncontents"));
 	dconsole->priv->clear_xml_button = button;
 	gtk_box_pack_start (GTK_BOX (bbox), button, FALSE, FALSE, 0);
 	g_signal_connect (button, "clicked",
 			  G_CALLBACK (editor_clear_clicked_cb), dconsole);
 
-	button = browser_make_small_button (FALSE, _("Add data source"), GTK_STOCK_ADD,
+	button = browser_make_small_button (FALSE, TRUE, _("Add"), GTK_STOCK_ADD,
 					    _("Add a new data source"));
 	dconsole->priv->add_source_button = button;
 	gtk_box_pack_start (GTK_BOX (bbox), button, FALSE, FALSE, 0);
 	g_signal_connect (button, "clicked",
 			  G_CALLBACK (add_source_clicked_cb), dconsole);
 
-	button = browser_make_small_button (TRUE, _("Variables"), NULL, _("Show variables needed"));
+	button = browser_make_small_button (TRUE, FALSE, _("Variables"), NULL, _("Show variables needed"));
 	gtk_box_pack_start (GTK_BOX (bbox), button, FALSE, FALSE, 0);
 	dconsole->priv->params_toggle = GTK_TOGGLE_BUTTON (button);
 	g_signal_connect (button, "toggled",
 			  G_CALLBACK (variables_clicked_cb), dconsole);
 
-	button = browser_make_small_button (FALSE, _("Execute"), GTK_STOCK_EXECUTE, _("Execute specified\n"
+	button = browser_make_small_button (FALSE, FALSE, _("Execute"), GTK_STOCK_EXECUTE, _("Execute specified\n"
 										      "data manager"));
 	gtk_box_pack_start (GTK_BOX (bbox), button, FALSE, FALSE, 0);
 	g_signal_connect (button, "clicked",
 			  G_CALLBACK (execute_clicked_cb), dconsole);
 
-	button = browser_make_small_button (TRUE, _("View XML"), NULL, _("View specifications\n"
+	button = browser_make_small_button (TRUE, FALSE, _("View XML"), NULL, _("View specifications\n"
 									 "as XML (advanced)"));
 	gtk_box_pack_start (GTK_BOX (bbox), button, FALSE, FALSE, 0);
 	spec_editor_toggled_cb (GTK_TOGGLE_BUTTON (button), dconsole);
@@ -415,7 +420,7 @@ data_console_new (BrowserConnection *bcnc)
 			  G_CALLBACK (spec_editor_toggled_cb), dconsole);
 
 #ifdef HAVE_GDU
-	button = browser_make_small_button (FALSE, _("Help"), GTK_STOCK_HELP, _("Help"));
+	button = browser_make_small_button (FALSE, FALSE, _("Help"), GTK_STOCK_HELP, _("Help"));
 	gtk_box_pack_start (GTK_BOX (bbox), button, FALSE, FALSE, 0);
 	g_signal_connect (button, "clicked",
 			  G_CALLBACK (help_clicked_cb), dconsole);
@@ -614,14 +619,14 @@ save_clicked_cb (GtkWidget *button, DataConsole *dconsole)
 }
 
 static void
-execute_clicked_cb (GtkButton *button, DataConsole *dconsole)
+execute_clicked_cb (G_GNUC_UNUSED GtkButton *button, DataConsole *dconsole)
 {
 	data_console_execute (dconsole);
 }
 
 #ifdef HAVE_GDU
 static void
-help_clicked_cb (GtkButton *button, DataConsole *dconsole)
+help_clicked_cb (G_GNUC_UNUSED GtkButton *button, DataConsole *dconsole)
 {
 	browser_show_help ((GtkWindow*) gtk_widget_get_toplevel ((GtkWidget*) dconsole),
 			   "data-manager-perspective");
@@ -659,7 +664,7 @@ spec_editor_toggled_cb (GtkToggleButton *button, DataConsole *dconsole)
 }
 
 static void
-param_activated_cb (GdauiBasicForm *form, DataConsole *dconsole)
+param_activated_cb (G_GNUC_UNUSED GdauiBasicForm *form, DataConsole *dconsole)
 {
 	DataWidget *dwid = NULL;
 	if (dconsole->priv->data)
@@ -669,7 +674,8 @@ param_activated_cb (GdauiBasicForm *form, DataConsole *dconsole)
 }
 
 static void
-data_source_source_changed_cb (DataSourceManager *mgr, DataSource *source, DataConsole *dconsole)
+data_source_source_changed_cb (DataSourceManager *mgr, G_GNUC_UNUSED DataSource *source,
+			       DataConsole *dconsole)
 {
 	data_source_mgr_changed_cb (mgr, dconsole);
 }
@@ -703,7 +709,7 @@ data_source_mgr_changed_cb (DataSourceManager *mgr, DataConsole *dconsole)
 }
 
 static void
-editor_clear_clicked_cb (GtkButton *button, DataConsole *dconsole)
+editor_clear_clicked_cb (G_GNUC_UNUSED GtkButton *button, DataConsole *dconsole)
 {
 	xml_spec_editor_set_xml_text (XML_SPEC_EDITOR (dconsole->priv->xml_sped), DEFAULT_XML);
 	gtk_widget_grab_focus (dconsole->priv->xml_sped);
@@ -718,7 +724,6 @@ add_source_mitem_activated_cb (GtkMenuItem *mitem, DataConsole *dconsole)
 	GSList *list;
 
 	table = (gchar*) g_object_get_data ((GObject*) mitem, "_table");
-	g_print ("Add data source for [%s]\n", table);
 
 	source = data_source_new (dconsole->priv->bcnc, DATA_SOURCE_UNKNOWN);
 	list = (GSList*) data_source_manager_get_sources (dconsole->priv->mgr);
@@ -734,11 +739,32 @@ add_source_mitem_activated_cb (GtkMenuItem *mitem, DataConsole *dconsole)
 	g_object_unref (source);
 }
 
+static gint
+dbo_sort (GdaMetaDbObject *dbo1, GdaMetaDbObject *dbo2)
+{
+	gint res;
+	res = g_strcmp0 (dbo1->obj_schema, dbo2->obj_schema);
+	if (res)
+		return - res;
+	return g_strcmp0 (dbo1->obj_name, dbo2->obj_name);
+}
+
 static void
-add_source_clicked_cb (GtkButton *button, DataConsole *dconsole)
+add_source_clicked_cb (G_GNUC_UNUSED GtkButton *button, DataConsole *dconsole)
 {
 	GdaMetaStruct *mstruct;
+	DataSource *current_source;
+
 	mstruct = browser_connection_get_meta_struct (dconsole->priv->bcnc);
+	current_source = ui_spec_editor_get_selected_source (UI_SPEC_EDITOR (dconsole->priv->ui_sped));
+
+	/* remove customization, if any */
+	if (dconsole->priv->customized_menu_items_list) {
+		g_slist_foreach (dconsole->priv->customized_menu_items_list,
+				 (GFunc) gtk_widget_destroy, NULL);
+		g_slist_free (dconsole->priv->customized_menu_items_list);
+		dconsole->priv->customized_menu_items_list = NULL;
+	}
 
 	if (dconsole->priv->add_source_menu &&
 	    (dconsole->priv->add_source_menu_index != (gpointer) mstruct)) {
@@ -747,15 +773,20 @@ add_source_clicked_cb (GtkButton *button, DataConsole *dconsole)
 		dconsole->priv->add_source_menu_index = NULL;
 	}
 	if (dconsole->priv->add_source_menu) {
+		if (current_source)
+			dconsole->priv->customized_menu_items_list =
+				data_manager_add_proposals_to_menu (dconsole->priv->add_source_menu,
+								    dconsole->priv->mgr,
+								    current_source, GTK_WIDGET (dconsole));
 		gtk_menu_popup (GTK_MENU (dconsole->priv->add_source_menu), NULL, NULL,
-			NULL, NULL, 0,
-			gtk_get_current_event_time ());
+				NULL, NULL, 0,
+				gtk_get_current_event_time ());
 		return;
 	}
 
-	GtkWidget *menu, *mitem;
+	GtkWidget *menu, *mitem, *submenu = NULL;
 	menu = gtk_menu_new ();
-	mitem = gtk_menu_item_new_with_label (_("Data source from query"));
+	mitem = gtk_menu_item_new_with_label (_("Data source from SQL"));
 	g_signal_connect (mitem, "activate",
 			  G_CALLBACK (add_source_mitem_activated_cb), dconsole);
 	gtk_widget_show (mitem);
@@ -764,7 +795,11 @@ add_source_clicked_cb (GtkButton *button, DataConsole *dconsole)
 	if (mstruct) {
 		gboolean sep_added = FALSE;
 		GSList *dbo_list, *list;
+		gchar *current_schema = NULL;
+
 		dbo_list = gda_meta_struct_get_all_db_objects (mstruct);
+		list = g_slist_sort (dbo_list, (GCompareFunc) dbo_sort);
+		dbo_list = list;
 		for (list = dbo_list; list; list = list->next) {
 			GdaMetaDbObject *dbo = GDA_META_DB_OBJECT (list->data);
 			gchar *str;
@@ -776,57 +811,60 @@ add_source_clicked_cb (GtkButton *button, DataConsole *dconsole)
 				gtk_menu_shell_append (GTK_MENU_SHELL (menu), mitem);
 				sep_added = TRUE;
 			}
-			str = g_strdup_printf (_("For table: %s"), dbo->obj_short_name);
+
+			if (!strcmp (dbo->obj_short_name, dbo->obj_full_name)) {
+				gboolean schema_changed = TRUE;
+				if (!current_schema)
+					current_schema = g_strdup (dbo->obj_schema);
+				else if (strcmp (current_schema, dbo->obj_schema)) {
+					g_free (current_schema);
+					current_schema = g_strdup (dbo->obj_schema);
+				}
+				else
+					schema_changed = FALSE;
+
+				if (schema_changed) {
+					str = g_strdup_printf (_("In schema '%s'"), current_schema);
+					mitem = gtk_menu_item_new_with_label (str);
+					gtk_widget_show (mitem);
+					g_free (str);
+					gtk_menu_shell_append (GTK_MENU_SHELL (menu), mitem);
+					submenu = gtk_menu_new ();
+					gtk_menu_item_set_submenu (GTK_MENU_ITEM (mitem), submenu);
+				}
+			}
+
+			str = g_strdup_printf (_("For table: %s"), dbo->obj_name);
 			mitem = gtk_menu_item_new_with_label (str);
 			g_object_set_data_full ((GObject*) mitem, "_table",
 						g_strdup (dbo->obj_short_name), g_free);
 			g_signal_connect (mitem, "activate",
 					  G_CALLBACK (add_source_mitem_activated_cb), dconsole);
 			gtk_widget_show (mitem);
-			gtk_menu_shell_append (GTK_MENU_SHELL (menu), mitem);			
+			gtk_menu_shell_append (GTK_MENU_SHELL (submenu ? submenu : menu), mitem);
 		}
 		g_slist_free (dbo_list);
+		g_free (current_schema);
 	}
 	dconsole->priv->add_source_menu_index = (gpointer) mstruct;
 	dconsole->priv->add_source_menu = menu;
+
+	if (current_source)
+		dconsole->priv->customized_menu_items_list =
+			data_manager_add_proposals_to_menu (dconsole->priv->add_source_menu,
+							    dconsole->priv->mgr,
+							    current_source, GTK_WIDGET (dconsole));
+
 	gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
 			NULL, NULL, 0,
 			gtk_get_current_event_time ());
 }
 
-static GtkWidget *
-create_widget (DataConsole *dconsole, GArray *sources_array, GError **error)
-{
-	GtkWidget *sw, *vp, *dwid;
-
-	if (! sources_array) {
-		g_set_error (error, 0, 0,
-			     _("No data source defined"));
-		return NULL;
-	}
-
-	sw = gtk_scrolled_window_new (NULL, NULL);
-	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
-					GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw), GTK_SHADOW_NONE);
-
-	vp = gtk_viewport_new (NULL, NULL);
-	gtk_viewport_set_shadow_type (GTK_VIEWPORT (vp), GTK_SHADOW_NONE);
-	gtk_container_add (GTK_CONTAINER (sw), vp);
-
-	dwid = data_widget_new (sources_array);
-	gtk_container_add (GTK_CONTAINER (vp), dwid);
-	g_object_set_data ((GObject*) sw, "data-widget", dwid);
-
-	gtk_widget_show_all (vp);
-	return sw;
-}
-
 /*
  * UI actions
  */
 static void
-compose_mode_toggled_cb (GtkToggleAction *action, DataConsole *dconsole)
+compose_mode_toggled_cb (G_GNUC_UNUSED GtkToggleAction *action, DataConsole *dconsole)
 {
 	gint pagenb;
 
@@ -843,36 +881,33 @@ compose_mode_toggled_cb (GtkToggleAction *action, DataConsole *dconsole)
 	pagenb = gtk_notebook_get_current_page (GTK_NOTEBOOK (dconsole->priv->main_notebook));
 	if (pagenb == MAIN_PAGE_EDITORS) {
 		/* Get Data sources */
-		GArray *sources_array;
-		GError *lerror = NULL;
-		sources_array = data_source_manager_get_sources_array (dconsole->priv->mgr, &lerror);
-		if (sources_array) {
-			if (dconsole->priv->data) {
-				/* destroy existing data widgets */
-				gtk_widget_destroy (dconsole->priv->data);
-				dconsole->priv->data = NULL;
-			}
+		pagenb = MAIN_PAGE_DATA;
+		if (dconsole->priv->data) {
+			/* destroy existing data widgets */
+			gtk_widget_destroy (dconsole->priv->data);
+			dconsole->priv->data = NULL;
+		}
 
-			GtkWidget *wid;
-			wid = create_widget (dconsole, sources_array, &lerror);
-			data_source_manager_destroy_sources_array (sources_array);
-			if (wid) {
-				dconsole->priv->data = wid;
-				gtk_box_pack_start (GTK_BOX (dconsole->priv->data_box), wid, TRUE, TRUE, 0);
-				gtk_widget_show (wid);
-				pagenb = MAIN_PAGE_DATA;
-			}
-		}
-		if (lerror) {
-			browser_show_error ((GtkWindow*) gtk_widget_get_toplevel ((GtkWidget*) dconsole),
-					    lerror && lerror->message ? lerror->message :
-					    _("Error parsing XML specifications"));
-			g_clear_error (&lerror);
-		}
-		if (pagenb == MAIN_PAGE_EDITORS) {
-			dconsole->priv->toggling = TRUE;
-			gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), TRUE);
-		}
+		GtkWidget *sw, *vp, *dwid;
+		
+		sw = gtk_scrolled_window_new (NULL, NULL);
+		gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
+						GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+		gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw), GTK_SHADOW_NONE);
+		
+		vp = gtk_viewport_new (NULL, NULL);
+		gtk_viewport_set_shadow_type (GTK_VIEWPORT (vp), GTK_SHADOW_NONE);
+		gtk_container_add (GTK_CONTAINER (sw), vp);
+		
+		dwid = data_widget_new (dconsole->priv->mgr);
+		gtk_container_add (GTK_CONTAINER (vp), dwid);
+		g_object_set_data ((GObject*) sw, "data-widget", dwid);
+		
+		gtk_widget_show_all (vp);
+
+		dconsole->priv->data = sw;
+		gtk_box_pack_start (GTK_BOX (dconsole->priv->data_box), sw, TRUE, TRUE, 0);
+		gtk_widget_show (sw);
 	}
 	else {
 		/* simply change the current page */
@@ -880,10 +915,11 @@ compose_mode_toggled_cb (GtkToggleAction *action, DataConsole *dconsole)
 	}
 
 	if (pagenb == MAIN_PAGE_DATA)
-		browser_show_notice ((GtkWindow*) gtk_widget_get_toplevel ((GtkWidget*) dconsole),
-				     "data-manager-exec-mode-switched",
-				     _("Switching to execution mode.\n\nHit the Escape key\n"
-				       "to return to the compose mode"));
+		browser_window_show_notice_printf (BROWSER_WINDOW (gtk_widget_get_toplevel ((GtkWidget*) dconsole)),
+						   GTK_MESSAGE_INFO,
+						   "data-manager-exec-mode-switched",
+						   "%s", _("Switching to execution mode. Hit the Escape key "
+						     "to return to the compose mode"));
 	gtk_notebook_set_current_page (GTK_NOTEBOOK (dconsole->priv->main_notebook), pagenb);
 }
 
@@ -913,6 +949,7 @@ data_console_page_get_actions_group (BrowserPage *page)
 	dconsole = DATA_CONSOLE (page);
 	if (! dconsole->priv->agroup) {
 		dconsole->priv->agroup = gtk_action_group_new ("QueryExecConsoleActions");
+		gtk_action_group_set_translation_domain (dconsole->priv->agroup, GETTEXT_PACKAGE);
 		gtk_action_group_add_toggle_actions (dconsole->priv->agroup,
 						     ui_actions, G_N_ELEMENTS (ui_actions), page);
 	}
@@ -920,7 +957,7 @@ data_console_page_get_actions_group (BrowserPage *page)
 }
 
 static const gchar *
-data_console_page_get_actions_ui (BrowserPage *page)
+data_console_page_get_actions_ui (G_GNUC_UNUSED BrowserPage *page)
 {
 	return ui_actions_console;
 }
